//--- install necessary functions

//--- RIF
ssc install rif

//--- storing regression results
ssc install estout
ssc install estwrite
help estout

//--- plotting estimates
ssc install coefplot
help coefplot
// http://repec.sowi.unibe.ch/stata/estout/index.html

***************************************
*** Unconditional Quantile?
***************************************

clear
set obs 1000
set seed 99999

gen x = runiform()
gen y = .

local x2 .1 .2 .3 .4 .5 .6 .7 .8 .9 1
local x1 0 .1 .2 .3 .4 .5 .6 .7 .8 .9
local xn : word count `x1'

local v1
local v2
forvalues i = 1/`xn' {
	local bg1 = `i' + 20 * `i' * 6
	local bg2 = `bg1' + 20 * `i' * 10
	
	local v1 `v1' `bg1'
	local v2 `v2' `bg2'
}
di "`v2'"

forvalues i = 1/`xn' {
	local low : word `i' of `x1'
	local upp : word `i' of `x2'
	
	local lowv : word `i' of `v1'
	local uppv : word `i' of `v2' 
	
	replace y = runiform(`lowv', `uppv') if x >= `low' & x < `upp'
}

sample 10 if x < .2
sample 20 if x >= .2 & x < .4
sample 30 if x >= .4 & x < .6
sample 40 if x >= .6 & x < .8
sample 50 if x >= .8 & x < 1

replace y = y/1000

qreg y x , q(90)
predict cqr90
qreg y x , q(50)
predict cqr50
qreg y x , q(10)
predict cqr10

sum y, d
gen uqr90 = `r(p90)' 
gen uqr50 = `r(p50)' 
gen uqr10 = `r(p10)' 

twoway ///
	(scatter y x, msize(1) fcolor(black) lcolor(black)) ///
	(lfit cqr90 x, lpattern(solid) lcolor(green)) ///
	(lfit cqr50 x, lpattern(solid) lcolor(green)) ///
	(lfit cqr10 x, lpattern(solid) lcolor(green)) ///
	, ///
	ytitle("Y") xtitle("X") ///
	title("(a) Conditional Quantiles", size(large)) ///
	legend(off) xsize(7) ysize(4) name(CQR, replace)

twoway ///
	(scatter y x, msize(1) fcolor(black) lcolor(black)) ///
	(lfit uqr90 x, lpattern(solid) lcolor(green)) ///
	(lfit uqr50 x, lpattern(solid) lcolor(green)) ///
	(lfit uqr10 x, lpattern(solid) lcolor(green)) ///
	, ///
	ytitle("Y") xtitle("X") ///
	title("(b) Unconditional Quantiles", size(large)) ///
	legend(off) xsize(7) ysize(4) name(UQR, replace)

gr combine CQR UQR, xsize(10) ysize(4) name(dist, replace)

********************************************************************************
*** Examples Using Stata
********************************************************************************

//- Data

use "../workingdata/QM0412", clear
desc
sum [aw=wt]

//- Control Variables

qui tab red, gen(educ)
qui tab mst, gen(mrst)
qui tab region, gen(rgnn)

isvar educ2-educ7 mrst2-mrst4 rgnn2-rgnn9
local ctrl age age2 nchild `r(varlist)'
center `ctrl', inplace

***************************************
*** UQR using RIF-Regression
***************************************

// (1) Calculate RIF for quantiles of log hourly wage, then OLS
forvalues q = 10(10)90 {
	egen hrwage_q`q' = rifvar(hrwage), q(`q') weight(wt)
}

forvalues q = 10(10)90 {
	qui reg hrwage_q`q' i.fem `ctrl' [pw=wt]
	eststo ols_q`q'
}

// (2) Use rifhdreg
forvalues q = 10(10)90 {
	qui rifhdreg hrwage i.fem `ctrl' [pw=wt], rif(q(`q'))
	eststo rif_q`q'
}

// Results

esttab ols_q10 ols_q50 ols_q90 rif_q10 rif_q50 rif_q90, ///
	mtitle(ols_q10 ols_q50 ols_q90 rif_q10 rif_q50 rif_q90) ///
	b(3) se(3) varwidth(10) lab noobs keep(*.fem)

***************************************
*** Obtain QTE
***************************************

// (1) Calculate IPW, Obtain RIF, OLS
texdoc stlog, beamer
qui logit fem `ctrl' [pw=wt]
qui predict IPW0
qui gen     IPW = .
qui replace IPW = 1/IPW0     if fem == 1
qui replace IPW = 1/(1-IPW0) if fem == 0
qui gen     IPWwt = wt * IPW

forvalues q = 10(10)90 {
	egen hrwage_q`q'_ipw = rifvar(hrwage), q(`q') weight(IPWwt) by(fem)
}

forvalues q = 10(10)90 {
	qui reg hrwage_q`q'_ipw i.fem `ctrl' [pw=IPWwt]
	eststo olsIPW_q`q'
}

// (2) Use rifhdreg
forvalues q = 10(10)90 {
	qui rifhdreg hrwage i.fem `ctrl' [pw=wt], rif(q(`q')) over(fem) rwlogit(`ctrl') ate
	eststo ate_q`q'
}

// QTE Results
esttab olsIPW_q10 olsIPW_q50 olsIPW_q90 ate_q10 ate_q50 ate_q90, ///
	mtitle(olsIPW_q10 olsIPW_q50 olsIPW_q90 ate_q10 ate_q50 ate_q90) ///
	b(3) se(3) varwidth(10) lab noobs keep(*.fem)


***************************************
*** QTE Results Comparison
***************************************

capt prog drop mergemodels
prog mergemodels, eclass
	// assuming that last element in e(b)/e(V) is not _cons
	version 8
	syntax namelist
	tempname b V tmp
	foreach name of local namelist {
		qui est restore `name'
		mat `b' = nullmat(`b') , e(b)
		mat `b' = `b'[1,1..colsof(`b')]
		mat `tmp' = e(V)
		mat `tmp' = `tmp'[1..rowsof(`tmp'),1..colsof(`tmp')]
		capt confirm matrix `V'
		if _rc {
			mat `V' = `tmp'
		}
		else {
			mat `V' = ///
				( `V' , J(rowsof(`V'),colsof(`tmp'),0) ) \ ///
				( J(rowsof(`tmp'),colsof(`V'),0) , `tmp' )
		}
	}
	local names: colfullnames `b'
	mat coln `V' = `names'
	mat rown `V' = `names'
	eret post `b' `V'
	eret local cmd "whatever"
end

capt prog drop matest
prog matest
	local yr `1' 
	mat b =  `2'
	mat v =  `3'
	mat colnames b = `yr'
	mat colnames v = `yr'
	mat rownames v = `yr'
	ereturn post b v
end

foreach md in ols rif ate {
	forvalues q = 10(10)90 {
		est res `md'_q`q'

		local fem_coln = colnumb(e(b), "1.fem")
		local fem_rown = colnumb(e(V), "1.fem")

		mat b = e(b)[1, `fem_coln']
		mat v = e(V)[`fem_rown', `fem_coln']
		
		mat colnames b = "q`q'"
		mat colnames v = "q`q'"
		mat rownames v = "q`q'"

		ereturn post b v
		eststo q`q'
	}
	mergemodels q10 q20 q30 q40 q50 q60 q70 q80 q90
	eststo `md'
}

mat D = (10, 20, 30, 40, 50, 60, 70, 80, 90)

coefplot rif ate ///
	, vert recast(connected) nooffset ///
	xtitle("Unconditional Quantile of Log Hourly Wage") ///
	ytitle("Coefficient") ///
	ciopts(recast(rarea) fcolor(%20) lcolor(%0)) ///
	xlabel(10 20 30 40 50 60 70 80 90) ///
	at(mat(D)) ///
	legend(order(2 "RIF" 4 "RIF-QTE") rows(1) pos(6)) ///
	xsize(7) ysize(4) name(comparison1, replace)

***************************************
*** RIF Based Oaxaca-Blinder Decomposition
***************************************

local ctro (age: age age2) (educ: educ2-educ7) (mrst: mrst2-mrst4) (rgnn: rgnn2-rgnn9) nchild

forvalues q = 10(10)90 {
	qui oaxaca_rif hrwage `ctro' [pw=wt], rif(q(`q')) by(fem) swap wgt(1) rwlogit(`ctrl')
	eststo decomp_q`q'
}

// Results
esttab decomp_q10 decomp_q30 decomp_q50 decomp_q70 decomp_q90, mtitle(q10 q30 q50 q70 q90) ///
	b(3) nose not keep(Overall:* explained:* unexplained:*) noobs

// Graph
forvalues q = 10(10)90 {
	est res decomp_q`q'

	local coln = colnumb(e(b), "Overall:tdifference")
	local rown = rownumb(e(V), "Overall:tdifference")

	mat b = e(b)[1, `coln']
	mat v = e(V)[`rown', `coln']

	mat colnames b = "q`q'"
	mat colnames v = "q`q'"
	mat rownames v = "q`q'"

	ereturn post b v
	eststo q`q'
}
mergemodels q10 q20 q30 q40 q50 q60 q70 q80 q90
eststo tot

forvalues q = 10(10)90 {
	est res decomp_q`q'

	local coln = colnumb(e(b), "explained:p_explained")
	local rown = rownumb(e(V), "explained:p_explained")

	mat b = e(b)[1, `coln']
	mat v = e(V)[`rown', `coln']

	mat colnames b = "q`q'"
	mat colnames v = "q`q'"
	mat rownames v = "q`q'"

	ereturn post b v
	eststo q`q'
}
mergemodels q10 q20 q30 q40 q50 q60 q70 q80 q90
eststo comp

forvalues q = 10(10)90 {
	est res decomp_q`q'

	local coln = colnumb(e(b), "unexplained:p_unexplained")
	local rown = rownumb(e(V), "unexplained:p_unexplained")

	mat b = e(b)[1, `coln']
	mat v = e(V)[`rown', `coln']

	mat colnames b = "q`q'"
	mat colnames v = "q`q'"
	mat rownames v = "q`q'"

	ereturn post b v
	eststo q`q'
}
mergemodels q10 q20 q30 q40 q50 q60 q70 q80 q90
eststo coef

coefplot tot comp coef ///
	, vert recast(connected) nooffset ///
	xtitle("Unconditional Quantile of Log Hourly Wage") ///
	ciopts(recast(rarea) fcolor(%20) lcolor(%0)) ///
	xlabel(10 20 30 40 50 60 70 80 90) ///
	at(mat(D)) ///
	legend(order(2 "Total" 4 "Composition" 6 "Coefficient") rows(1) pos(6)) ///
	xsize(7) ysize(4) name(decomp, replace)
